#ifndef RANKERS_H_
#define RANKERS_H_

#include "../PrecedenceRulesGraph.h"
#include "IGraphVisitor.h"

namespace concurrency {

  //--------------------------------------------------------------------------
  class RankerByProductConsumption : public IGraphVisitor {
  public:
    using IGraphVisitor::visit;

    bool visit( AlgorithmNode& ) override;
  };

  //--------------------------------------------------------------------------
  class RankerByCummulativeOutDegree : public IGraphVisitor {
  public:
    using IGraphVisitor::visit;

    bool visit( AlgorithmNode& ) override;
    void reset() override { m_nodesSucceeded = 0; };

    void runThroughAdjacents( boost::graph_traits<precedence::PrecTrace>::vertex_descriptor, precedence::PrecTrace );

    int m_nodesSucceeded{0};
  };

  //--------------------------------------------------------------------------
  class RankerByTiming : public IGraphVisitor {
  public:
    using IGraphVisitor::visit;

    bool visit( AlgorithmNode& ) override;
  };

  //--------------------------------------------------------------------------
  class RankerByEccentricity : public IGraphVisitor {
  public:
    using IGraphVisitor::visit;

    bool visit( AlgorithmNode& ) override;
  };

  //--------------------------------------------------------------------------
  class RankerByDataRealmEccentricity : public IGraphVisitor {
  public:
    using IGraphVisitor::visit;

    bool visit( AlgorithmNode& ) override;

    void reset() override {
      m_currentDepth  = 0;
      m_maxKnownDepth = 0;
    }

    /// Depth-first node parser to calculate node eccentricity
    /// (only within the data realm of the precedence rules graph)
    void recursiveVisit( AlgorithmNode& );

    uint m_currentDepth{0};
    uint m_maxKnownDepth{0};
  };

} // namespace concurrency

#endif /* RANKERS_H_ */
